"""
This is a user-friendly script for generating a pseudotrajectory.
"""

import argparse
from os.path import exists

from ..paths import PATH_INPUT_BASEGRO
from molgri.parsers import NameParser, TranslationParser
from ..scripts.generate_grid import prepare_grid
from ..scripts.set_up_io import freshly_create_all_folders

# TODO: define total_N and generate in all dimensions uniform grid?
# TODO: allow different rotation grids for two types of rotation
from molgri.pts import Pseudotrajectory

parser = argparse.ArgumentParser()
requiredNamed = parser.add_argument_group('required named arguments')
requiredNamed.add_argument('-m1', type=str, nargs='?', required=True,
                    help='name of the .gro file containing the fixed molecule')
requiredNamed.add_argument('-m2', type=str, nargs='?', required=True,
                    help='name of the .gro file containing the mobile molecule')
requiredNamed.add_argument('-rotgrid', metavar='rg', type=str, nargs='?', required=True,
                    help='name of the rotation grid in the form algorithm_N (eg. ico_50)')
requiredNamed.add_argument('-transgrid', metavar='tg', type=str, nargs='?', required=True,
                    help='translation grid provided as a list of distances, as linspace(start, stop, num) '
                         'or range(start, stop, step) in nanometers')
parser.add_argument('--only_origin', action='store_true',
                    help='only include rotations around the origin, not around the body')
parser.add_argument('--recalculate', action='store_true',
                    help='recalculate the grid even if a saved version already exists')


def check_file_existence(args):
    path_c_mol = f"{PATH_INPUT_BASEGRO}{args.m1}.gro"
    path_r_mol = f"{PATH_INPUT_BASEGRO}{args.m2}.gro"
    # check if rotgrid name a valid name
    nap = NameParser(args.rotgrid)
    N = nap.num_grid_points
    algo = nap.grid_type
    assert algo is not None, f"Rotation grid algorithm not recognised, check rotgrid argument: {args.rotgrid}"
    assert N is not None, f"Num of grid points not recognised, check rotgrid argument: {args.rotgrid}"
    # check if input rotgrid files exist
    if not exists(path_c_mol):
        raise FileNotFoundError(f"Could not find the file {args.m1}.gro at {PATH_INPUT_BASEGRO}. "
                                "Please provide a valid .gro file name as the first script parameter.")
    if not exists(path_r_mol):
        raise FileNotFoundError(f"Could not find the file {args.m2}.gro at {PATH_INPUT_BASEGRO}. "
                                "Please provide a valid .gro file name as the second script parameter.")
    # if the rotational grid file doesn't exist, create it
    rot_grid = prepare_grid(args, nap)
    # parse translational grid
    trans_grid = TranslationParser(args.transgrid)
    return rot_grid, trans_grid


def prepare_pseudotrajectory(args, r_grid, t_grid):
    if args.only_origin:
        traj_type = "circular"
    else:
        traj_type = "full"
    pt = Pseudotrajectory(args.m1, args.m2, rot_grid=r_grid, trans_grid=t_grid, traj_type=traj_type)
    end_index = pt.generate_pt_and_time()
    print(f"Generated a {pt.decorator_label} with {end_index} timesteps.")


def run_generate_pt():
    freshly_create_all_folders()
    my_args = parser.parse_args()
    my_rg, my_tg = check_file_existence(my_args)
    prepare_pseudotrajectory(my_args, my_rg, my_tg)


if __name__ == '__main__':
    run_generate_pt()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scripts']

package_data = \
{'': ['*']}

install_requires = \
['requires-python>=3.6,<4.0']

entry_points = \
{'console_scripts': ['molgri-grid = '
                     'molgri.scripts.generate_grid:run_generate_grid',
                     'molgri-io = molgri.scripts.set_up_io:parse_and_create',
                     'molgri-pt = molgri.scripts.generate_pt:run_generate_pt']}

setup_kwargs = {
    'name': 'molgri',
    'version': '1.0.1',
    'description': 'Generate molecular pseudotrajectories based on rotational grids.',
    'long_description': '# molecularRotationalGrids\nThe python package ```molgri``` has three main purposes: 1) generation of rotation grids, 2) analysis of\nsaid grids and 3) generation of pseudotrajectories (PTs). PTs are .gro files with several timesteps in\nwhich the interaction space of two molecules is systematically explored. We provide user-friendly,\none-line scripts for rotation grid generation and analysis as well as pseudotrajectory generation and\ngive instructions how to use PTS with external tools like VMD and GROMACS for further analysis.\n\nBelow, we show examples of rotation grids created with different algorithms and figures of\na protein-ion pseudotrajectory as well as some analysis plots. All plots and animations are created\ndirectly with the ```molgri``` package, except the PT plot where the output of ```molgri``` is drawn\nusing VMD.\n\n\n<p float="left">\n    <img src="/readme_images/ico_630_grid.gif" width="48%">\n    <img src="/readme_images/systemE_1000_grid.gif" width="48%">\n</p>\n\n<p float="left">\n    <img src="/readme_images/systemE_1000_uniformity.png" width="30%">\n    <img src="/readme_images/set_up_30_full_color.png" width="30%">\n    <img src="/readme_images/systemE_1000_convergence.png" width="30%">\n</p>\n\n\n\n## Installation\n\nThis is a python library that can be easily installed using:\n\n```\npip install molgri\n```\n\n\n\n## Running examples\n\nTo explore the capabilities of the package, the user is encouraged to run\nthe following example commands (the commands should all be executed in the\nsame directory, we recommend an initially empty directory).\n\n```\nmolgri-io --examples\nmolgri-grid -N 250 -algo ico --draw --animate --animate_ordering --statistics\nmolgri-pt -m1 H2O -m2 NH3 -rotgrid cube3D_15 -transgrid "range(1, 5, 2)"\n```\n\n****The first-line command ```molgri-io``` creates the 📂 input/ and\n📂 output/ folder structure. This command should be run in each new\ndirectory before running other commands. The optional\nflag ```--examples``` provides some sample input files that we will use later.\n\nThe second command ```molgri-grid``` is used to generate rotation grids. It is\nnecessary to specify the number of grid points ```-N``` and the algorithm \n```-algo``` (select from: systemE, randomE, randomQ, cube4D, cube3D, ico; we\nrecommend ico). Other flags describe optional figures and animations to save. All\ngenerated files can be found in the output/ folder.\n\nThe last command ```molgri-pt``` creates a pseudotrajectory. This scripts needs\ntwo file inputs that should be provided in input/. Both must be\n.gro files, each containing a single molecule. Due to the flag\n```-m1 H2O``` the program will look for a file input/H2O.gro\nand use it as a fixed molecule in the pseudotrajectory. The flag ```-m2```\ngives the name of the file with the other molecule, which will be mobile\nin the simulation. Finally, the user needs to specify the rotational grid\nin form ```-rotgrid algorithm_N``` (see algorithm names above) and the translational grid after the\nflag ```-transgrid``` in one of the\nfollowing formats: a list of distances (in nm), linspace(start, stop, num) \nor range(start, stop, step). The argument should be surrounded by quotation\nmarks. Some acceptable translation grid arguments would be:\n\n```-transgrid "(1, 3, 5)"``` -> use distances 1nm, 3nm and 5nm\n\n```-transgrid "linspace(1, 3, 5)"``` -> use 5 equally spaced points\nbetween 1nm and 3nm\n\n```-transgrid "range(1, 3, 0.5)"``` -> use distances between 1nm and 3nm\nin 0.5nm increments\n\nAll flags starting with ```--``` are optional and can be omitted for faster\ncalculations. Remember that you can always add the flag ```--help``` to get\nfurther instructions.****\n\n\n## Using outputs\n\nThe pseudotrajectory .gro files can be used as regularly generated .gro files. We show how they can be\ndisplayed with VMD or used for GROMACS calculations, but the user is free to use them as inputs to any other\ntool.\n\n#### Displaying pseudotrajectory\n\nTo display the example pseudotrajectory we created in the previous section with VMD, stay in the same\ndirectory and run\n\n```\nvmd output/pt_files/H2O_NH3_cube3D_15_full.gro\n```\n\nor on a windows computer\n\n```\nstart vmd output/pt_files/H2O_NH3_cube3D_15_full.gro\n```\n\nThen, to fully display a pseudotrajectory, it is often helpful to change the display style and to display\nseveral or all frames at once. We suggest using the following commands within the VMD command line:\n\n```\nmol modstyle 0 0 VDW\nmol drawframes 0 0 1:1:241\n```\n\nThe first one displays the molecules as spheres with van der Waals radii and the second draws frames of\nthe pseudotrajectory in a form &lt;start>:&lt;step>:&lt;stop>.\nA useful trick is to use the number of rotations as &lt;step> (in this case that would be 15) - this displays one structure per mass point\nwithout considering internal rotations. This number is also written in the name of the .gro file.\nIf you want to display all frames, you can use any large number\nfor &lt;num_frames>, it does not need to correspond exactly to the number of frames.\n\n#### Calculating energy along a pseudotrajectory\nOften, a pseudotrajectory is used to explore where reagions of high and low energies lie when molecules\napproach each other. Since a range of timesteps sampling important rotations and translations\nis already provided in a PT, there is no need to run a real\nsimulation. Therefore, the flag ```-rerun``` is always used while dealing with PTs in GROMACS. This\nsetting saves time that would else be used for running an integrator and propagating positions.\n\n\nTo use GROMACS with PTs, the user must also provide a topology file which includes both molecules used in a pseudotrajectory.\nWe will assume that this file is named topol.top. Lastly, we need a GROMACS run file that we will name\nmdrun.mdp. This file records GROMACS parameters and can be used as in normal simulations, but note that\nsome parameters (e.g. integrator, dt) are meaningless for a pseudotrajectory without actual dynamics.\nThen, the energy along a pseudotrajectory can be calculated as follows:\n\n```\ngmx22 grompp -f mdrun.mdp -c H2O_NH3_cube3D_15_full.gro -p topol.top -o result.tpr   \ngmx22 trjconv -f H2O_NH3_cube3D_15_full.gro -s result.tpr -o result.trr\ngmx22 mdrun -s result.tpr -rerun result.trr\ngmx22 energy -f ener.edr -o full_energy.xvg\n```\n\n\n## Complex applications: using python package\n\nUsers who would like to build custom grids,  pseudotrajectories or sets of rotations can import ```molgri```\nas a python package (following installation described above) and work with all provided modules. Documentation\nis provided in form of docstrings or available in a compiled version at our github repository \n[in docs folder](https://github.com/bkellerlab/molecularRotationalGrids/tree/main/docs/molgri).',
    'author': 'Hana Zupan',
    'author_email': 'hana.zupan@fu-berlin.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://www.bcp.fu-berlin.de/en/chemie/chemie/forschung/PhysTheoChem/agkeller/index.html',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
}


setup(**setup_kwargs)



"""
This script should be run to define input and output directories if the user wants to select directories other
than default ones. It can also be run at any later point to move the contents from previously selected directories
to new ones.
"""
import argparse
import os
import shutil

from ..constants import PATH_USER_PATHS, PATH_EXAMPLES

parser = argparse.ArgumentParser()
parser.add_argument('--examples', action='store_true',
                    help='copy example inputs')

# if you introduce new IO folders, add them to next three lines, keep the order the same!
IO_FOLDER_PURPUSES = ["rotational grids", "pseudotrajectory files", "base gro files", "plots", "animations",
                      "grid statistics"]
IO_FOLDER_DEFAULTS = ["output/grid_files/", "output/pt_files/", "input/", "output/figures/",
                      "output/animations/", "output/statistics_files/"]
IO_VARIABLE_NAMES = ["PATH_OUTPUT_ROTGRIDS", "PATH_OUTPUT_PT", "PATH_INPUT_BASEGRO", "PATH_OUTPUT_PLOTS",
                     "PATH_OUTPUT_ANIS", "PATH_OUTPUT_STAT"]


def parse_and_create():
    my_args = parser.parse_args()
    freshly_create_all_folders()
    # copy example inputs if option selected
    if my_args.examples:
        src_files = os.listdir(PATH_EXAMPLES)
        for file_name in src_files:
            full_file_name = os.path.join(PATH_EXAMPLES, file_name)
            if os.path.isfile(full_file_name) and full_file_name.endswith(".gro"):
                shutil.copy(full_file_name, IO_FOLDER_DEFAULTS[2])


def freshly_create_all_folders():
    for path in IO_FOLDER_DEFAULTS:
        if not os.path.exists(path):
            os.makedirs(path)
    with open(PATH_USER_PATHS, "w") as f:
        for varname, new_value in zip(IO_VARIABLE_NAMES, IO_FOLDER_DEFAULTS):
            f.write(f"{varname} = '{new_value}'\n")


if __name__ == '__main__':
    parse_and_create()
